package test;

import org.junit.runners.MethodSorters;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.BeforeClass;

import static org.junit.Assert.*;

import java.io.IOException;
import java.time.Instant;

import org.junit.AfterClass;

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class Tests {

	// START CONFIG
	static String ip = "127.0.0.1";
	static int port = 8080;
	// END CONFIG

	static CommunicationHandler c1 = new CommunicationHandler();
	static CommunicationHandler c2 = new CommunicationHandler();
	static CommunicationHandler c3 = new CommunicationHandler();

	@BeforeClass
	public static void connect() {
		boolean flawless = true;
		flawless &= c1.connectToSock(ip, port);
		flawless &= c2.connectToSock(ip, port);
		flawless &= c3.connectToSock(ip, port);

		assertTrue(flawless);
		sleep(1); // Welcome Messages Aufr�umen
		c1.getBuffer();
		c2.getBuffer();
		c3.getBuffer(); 
	}

	@AfterClass
	public static void disconnect() {
		c1.endThread(true);
		c2.endThread(true);
		c3.endThread(true);
	}

	@Test
	public void A_wEmpty1_Test_1() {
		System.out.println("Test 1");
		c1.send("W 10");
		sleep(1);
		String buf = c1.getBuffer();
		System.out.println("C1 sendet W 10");
		System.out.println("IS: " + buf + " SHOULD: 0 / E");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buf.trim().startsWith("E ") || buf.trim().equals("0"));
	}

	@Test
	public void B_lEmpty1_Test_2() {
		System.out.println("Test 2");
		c1.send("L 10");
		sleep(1);
		String buf = c1.getBuffer();
		System.out.println("C1 sendet L 10");
		System.out.println("IS: " + buf + " SHOULD: 0 / E");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buf.trim().startsWith("E ") || buf.trim().equals("0"));
	}

	@Test
	public void C_tEmpty1_Test_3() {
		System.out.println("Test 3");
		c1.send("T 10");
		sleep(1);
		String buf = c1.getBuffer();
		System.out.println("C1 sendet T 10");
		System.out.println("IS: " + buf + " SHOULD: 0 | E No Messages");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buf.trim().startsWith("E ") || buf.trim().equals("0"));
	}

	@Test
	public void D_singleP1_Test_4() {
		System.out.println("Test 4");
		c1.sendB("P\n1\n2\n0 Topic 0\nText 0");
		sleep(1);
		String buf1 = c1.getBuffer();
		String buf2 = c2.getBuffer();
		String buf3 = c3.getBuffer();
		System.out.println("C1 sendet P 1 2 0 Topic 0 Text 0");
		System.out.println("IS: " + buf1 + " SHOULD: N 1 t Topic 0");
		System.out.println("IS: " + buf2 + " SHOULD: N 1 t Topic 0");
		System.out.println("IS: " + buf3 + " SHOULD: N 1 t Topic 0");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buf1.split("\n")[0].trim().startsWith("N 1"));
		assertTrue(buf1.split("\n")[1].trim().endsWith(" Topic 0"));
		assertTrue(buf1.split("\n")[1].trim().charAt(0) != '0');
		assertTrue(buf2.split("\n")[0].trim().startsWith("N 1"));
		assertTrue(buf2.split("\n")[1].trim().endsWith(" Topic 0"));
		assertTrue(buf2.split("\n")[1].trim().charAt(0) != '0');
		assertTrue(buf3.split("\n")[0].trim().startsWith("N 1"));
		assertTrue(buf3.split("\n")[1].trim().endsWith(" Topic 0"));
		assertTrue(buf3.split("\n")[1].trim().charAt(0) != '0');
	}

	@Test
	public void E_singleP2_Test_5() {
		System.out.println("Test 5");
		c2.sendB("P\n1\n3\n1 Topic 1\nText 1\nText 1,2");
		sleep(1);
		String buf1 = c1.getBuffer();
		String buf2 = c2.getBuffer();
		String buf3 = c3.getBuffer();
		System.out.println("C2 sendet P 1 3 1 Topic 1 Text 1 Text 1,2");
		System.out.println("IS: " + buf1 + " SHOULD: N 1 t Topic 1");
		System.out.println("IS: " + buf2 + " SHOULD: N 1 t Topic 1");
		System.out.println("IS: " + buf3 + " SHOULD: N 1 t Topic 1");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buf1.split("\n")[0].trim().startsWith("N 1"));
		assertTrue(buf1.split("\n")[1].trim().endsWith(" Topic 1"));
		assertTrue(buf2.split("\n")[0].trim().startsWith("N 1"));
		assertTrue(buf2.split("\n")[1].trim().endsWith(" Topic 1"));
		assertTrue(buf3.split("\n")[0].trim().startsWith("N 1"));
		assertTrue(buf3.split("\n")[1].trim().endsWith(" Topic 1"));
	}

	@Test
	public void F_singleP3_Test_6() {
		System.out.println("Test 6");
		c3.sendB("P\n1\n4\n2 Topic 2\nText 2\nText 2,2\nText 2,3");
		sleep(1);
		String buf1 = c1.getBuffer();
		String buf2 = c2.getBuffer();
		String buf3 = c3.getBuffer();
		System.out.println("C3 sendet P 1 4 2 Topic 2 Text 2 Text 2,2 Text 2,3");
		System.out.println("IS: " + buf1 + " SHOULD: N 1 t Topic 2");
		System.out.println("IS: " + buf2 + " SHOULD: N 1 t Topic 2");
		System.out.println("IS: " + buf3 + " SHOULD: N 1 t Topic 2");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buf1.split("\n")[0].trim().startsWith("N 1"));
		assertTrue(buf1.split("\n")[1].trim().endsWith(" Topic 2"));
		assertTrue(buf1.split("\n")[1].trim().charAt(0) != '2');
		assertTrue(buf2.split("\n")[0].trim().startsWith("N 1"));
		assertTrue(buf2.split("\n")[1].trim().endsWith(" Topic 2"));
		assertTrue(buf2.split("\n")[1].trim().charAt(0) != '2');
		assertTrue(buf3.split("\n")[0].trim().startsWith("N 1"));
		assertTrue(buf3.split("\n")[1].trim().endsWith(" Topic 2"));
		assertTrue(buf3.split("\n")[1].trim().charAt(0) != '2');
	}

	@Test
	public void G_singleP4_Test_7() {
		System.out.println("Test 7");
		c3.sendB("P\n1\n2\n1 Topic 8\nText 8");
		sleep(1);
		String buf1 = c1.getBuffer();
		String buf2 = c2.getBuffer();
		String buf3 = c3.getBuffer();
		System.out.println("C3 sendet: P 1 2 1 Topic 8 Text 8");
		System.out.println("IS: " + buf1 + " SHOULD: N 1 t Topic 8");
		System.out.println("IS: " + buf2 + " SHOULD: N 1 t Topic 8");
		System.out.println("IS: " + buf3 + " SHOULD: N 1 t Topic 8");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buf1.split("\n")[0].trim().startsWith("N 1"));
		assertTrue(buf1.split("\n")[1].trim().endsWith(" Topic 8"));
		assertTrue(buf2.split("\n")[0].trim().startsWith("N 1"));
		assertTrue(buf2.split("\n")[1].trim().endsWith(" Topic 8"));
		assertTrue(buf3.split("\n")[0].trim().startsWith("N 1"));
		assertTrue(buf3.split("\n")[1].trim().endsWith(" Topic 8"));
	}

	@Test
	public void H_multiP1_Test_8() {
		System.out.println("Test 8");
		c1.sendB("P\n2\n2\n3 Topic 3\nText 3\n2\n4 Topic 4\nText 4");
		sleep(1);
		String buf1 = c1.getBuffer();
		String buf2 = c2.getBuffer();
		String buf3 = c3.getBuffer();
		System.out.println("C1 sendet P 2 2 3 Topic 3 Text 3 2 4 Topic 4 Text 4");
		System.out.println("IS: " + buf1 + " SHOULD: N 2 t Topic 3 t Topic 4");
		System.out.println("IS: " + buf2 + " SHOULD: N 2 t Topic 3 t Topic 4");
		System.out.println("IS: " + buf3 + " SHOULD: N 2 t Topic 3 t Topic 4");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buf1.startsWith("N 2"));
		assertTrue(buf1.split("\n")[1].trim().endsWith(" Topic 3"));
		assertTrue(buf1.split("\n")[2].trim().endsWith(" Topic 4"));
		assertTrue(buf1.split("\n")[1].trim().charAt(0) != '3');
		assertTrue(buf1.split("\n")[2].trim().charAt(0) != '4');
		assertTrue(buf2.startsWith("N 2"));
		assertTrue(buf2.split("\n")[1].trim().endsWith(" Topic 3"));
		assertTrue(buf2.split("\n")[2].trim().endsWith(" Topic 4"));
		assertTrue(buf2.split("\n")[1].trim().charAt(0) != '3');
		assertTrue(buf2.split("\n")[2].trim().charAt(0) != '4');
		assertTrue(buf3.startsWith("N 2"));
		assertTrue(buf3.split("\n")[1].trim().endsWith(" Topic 3"));
		assertTrue(buf3.split("\n")[2].trim().endsWith(" Topic 4"));
		assertTrue(buf3.split("\n")[1].trim().charAt(0) != '3');
		assertTrue(buf3.split("\n")[2].trim().charAt(0) != '4');
	}

	@Test
	public void I_multiP2_Test_9() {
		System.out.println("Test 9");
		c2.sendB("P\n3\n2\n5 Topic 5\nText 5\n2\n6 Topic 6\nText 6\n2\n7 Topic 7\nText 7");
		sleep(1);
		String buf1 = c1.getBuffer();
		String buf2 = c2.getBuffer();
		String buf3 = c3.getBuffer();
		System.out.println("C2 sendet P 3 2 5 Topic 5 Text 5 2 6 Topic 6 Text 6 2 7 Topic 7 Text 7");
		System.out.println("IS: " + buf1 + " SHOULD: N 3 t Topic 5 Topic 6 Topic 7");
		System.out.println("IS: " + buf2 + " SHOULD: N 3 t Topic 5 Topic 6 Topic 7");
		System.out.println("IS: " + buf3 + " SHOULD: N 3 t Topic 5 Topic 6 Topic 7");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buf1.startsWith("N 3"));
		assertTrue(buf1.split("\n")[1].trim().endsWith(" Topic 5"));
		assertTrue(buf1.split("\n")[2].trim().endsWith(" Topic 6"));
		assertTrue(buf1.split("\n")[3].trim().endsWith(" Topic 7"));
		assertTrue(buf1.split("\n")[1].trim().charAt(0) != '5');
		assertTrue(buf1.split("\n")[2].trim().charAt(0) != '6');
		assertTrue(buf1.split("\n")[3].trim().charAt(0) != '7');
		assertTrue(buf2.startsWith("N 3"));
		assertTrue(buf2.split("\n")[1].trim().endsWith(" Topic 5"));
		assertTrue(buf2.split("\n")[2].trim().endsWith(" Topic 6"));
		assertTrue(buf2.split("\n")[3].trim().endsWith(" Topic 7"));
		assertTrue(buf2.split("\n")[1].trim().charAt(0) != '5');
		assertTrue(buf2.split("\n")[2].trim().charAt(0) != '6');
		assertTrue(buf2.split("\n")[3].trim().charAt(0) != '7');
		assertTrue(buf3.startsWith("N 3"));
		assertTrue(buf3.split("\n")[1].trim().endsWith(" Topic 5"));
		assertTrue(buf3.split("\n")[2].trim().endsWith(" Topic 6"));
		assertTrue(buf3.split("\n")[3].trim().endsWith(" Topic 7"));
		assertTrue(buf3.split("\n")[1].trim().charAt(0) != '5');
		assertTrue(buf3.split("\n")[2].trim().charAt(0) != '6');
		assertTrue(buf3.split("\n")[3].trim().charAt(0) != '7');
	}

	@Test
	public void J_multiP3_Test_10() {
		System.out.println("Test 10");
		c3.sendB("P\n2\n2\n8 Topic 8\nText 8\n3\n9 Topic 9\nText 9\nText 9,2");
		sleep(1);
		String buf1 = c1.getBuffer();
		String buf2 = c2.getBuffer();
		String buf3 = c3.getBuffer();
		System.out.println("C3 sendet P 2 2 8 Topic 8 Text 8 3 9 Topic 9 Text 9 Text 9,2");
		System.out.println("IS: " + buf1 + " SHOULD: N 2 t Topic 8 t Topic 9");
		System.out.println("IS: " + buf2 + " SHOULD: N 2 t Topic 8 t Topic 9");
		System.out.println("IS: " + buf3 + " SHOULD: N 2 8 Topic 8 t Topic 9");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buf1.startsWith("N 2"));
		assertTrue(buf1.split("\n")[1].trim().endsWith(" Topic 8"));
		assertTrue(buf1.split("\n")[2].trim().endsWith(" Topic 9"));
		assertTrue(buf1.split("\n")[1].trim().charAt(0) != '8');
		assertTrue(buf1.split("\n")[2].trim().charAt(0) != '9');
		assertTrue(buf2.startsWith("N 2"));
		assertTrue(buf2.split("\n")[1].trim().endsWith(" Topic 8"));
		assertTrue(buf2.split("\n")[2].trim().endsWith(" Topic 9"));
		assertTrue(buf2.split("\n")[1].trim().charAt(0) != '8');
		assertTrue(buf2.split("\n")[2].trim().charAt(0) != '9');
		assertTrue(buf3.startsWith("N 2"));
		assertTrue(buf3.split("\n")[1].trim().endsWith(" Topic 8"));
		assertTrue(buf3.split("\n")[2].trim().endsWith(" Topic 9"));
		assertTrue(buf3.split("\n")[1].trim().charAt(0) != '8');
		assertTrue(buf3.split("\n")[2].trim().charAt(0) != '9');
	}

	@Test
	public void K_wEmpty2_Test_11() {
		System.out.println("Test 11");
		long unixTimestamp = Instant.now().getEpochSecond() + 10;
		c3.send("W " + unixTimestamp);
		sleep(1);
		String buf = c3.getBuffer();
		System.out.println("C1 sendet W " + unixTimestamp);
		System.out.println("IS: " + buf + " SHOULD: E / 0");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buf.startsWith("E ") || buf.trim().equals("0"));
	}

	@Test
	public void M_w2_Test_12() {
		System.out.println("Test 12");
		long unixTimestamp = Instant.now().getEpochSecond() - 1000;
		c3.send("W " + unixTimestamp);
		sleep(1);
		String buff = c3.getBuffer();

		System.out.println("C3 sendet W " + unixTimestamp);
		System.out.println("IS:" + buff + " SHOULD: 11 Topic 0, 1, 2, 8, 3, 4, 5, 6, 7, 8, 9");
		System.out.println("--------------------------------------------------------------------------------------");

		fail("Das m�sst ihr selbst �berpr�fen.");
	}

	@Test
	public void N_t1_Test_13() {
		System.out.println("Test 13");
		c3.send("T Topic 0");
		sleep(1);
		String buff = c3.getBuffer();
		System.out.println("C3 sendet T Topic 0");
		System.out.println("IS: " + buff + " SHOULD: 1 2 t Topic 0 Text 0");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buff.split("\n")[0].trim().endsWith("1"));
		assertTrue(buff.split("\n")[1].trim().endsWith("2"));
		assertTrue(buff.split("\n")[2].trim().endsWith(" Topic 0"));
		assertTrue(buff.split("\n")[2].trim().charAt(0) != '0');
		assertTrue(buff.split("\n")[3].trim().endsWith("Text 0"));
	}

	@Test
	public void O_t2_Test_14() {
		System.out.println("Test 14");
		c3.send("T Topic 8");
		sleep(1);
		String buff = c3.getBuffer();
		System.out.println("C3 sendet T Topic 8");
		System.out.println("IS: " + buff + " SHOULD: 2 2 t Topic 8 Text 8 2 t Topic 8 Text 8");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buff.split("\n")[0].trim().endsWith("2"));
		assertTrue(buff.split("\n")[1].trim().endsWith("2"));
		assertTrue(buff.split("\n")[2].trim().endsWith(" Topic 8"));
		assertTrue(buff.split("\n")[3].trim().endsWith("Text 8"));
		assertTrue(buff.split("\n")[4].trim().endsWith("2"));
		assertTrue(buff.split("\n")[5].trim().endsWith(" Topic 8"));
		assertTrue(buff.split("\n")[6].trim().endsWith("Text 8"));
	}

	@Test
	public void P_l0_Test_15() {
		System.out.println("Test 15");
		c3.send("L");
		sleep(1);
		String buf = c3.getBuffer();
		System.out.println("C3 sendet L");
		System.out.println("IS: " + buf
				+ " SHOULD: 10 t Topic 9 t Topic 8 t Topic 7 t Topic 6 t Topic 5 t Topic 4 t Topic 3 t Topic 2 t Topic 1 t Topic 0 (oder andere Reihenfolge)");
		System.out.println("--------------------------------------------------------------------------------------");

		fail("Das m�sst ihr selbst �berpr�fen.");
	}

	@Test
	public void Q_l1_Test_16() {
		System.out.println("Test 16");
		c3.send("L ");
		sleep(1);
		String buf = c3.getBuffer();
		System.out.println("C1 sendet L_");
		System.out.println("IS: " + buf
				+ " SHOULD: 10 t Topic 9 t Topic 8 t Topic 7 t Topic 6 t Topic 5 t Topic 4 t Topic 3 t Topic 2 t Topic 1 t Topic 0 (oder andere Reihenfolge)");
		System.out.println("--------------------------------------------------------------------------------------");

		fail("Das m�sst ihr selbst �berpr�fen.");
	}

	@Test
	public void R_l2_Test_17() {
		System.out.println("Test 17");
		c3.send("L 2");
		sleep(1);
		String buf = c3.getBuffer();
		System.out.println("C3 sendet L 2");
		System.out.println("IS: " + buf + " SHOULD: 2 t Topic 9 t Topic 8");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buf.split("\n")[0].trim().endsWith("2"));
		assertTrue(buf.split("\n")[1].trim().endsWith(" Topic 9") || buf.split("\n")[1].trim().endsWith(" Topic 8"));
		assertTrue(buf.split("\n")[2].trim().endsWith(" Topic 9") || buf.split("\n")[2].trim().endsWith(" Topic 8"));
		assertFalse(buf.split("\n")[1].trim().endsWith(" Topic 8") && buf.split("\n")[2].trim().endsWith(" Topic 8"));
		assertFalse(buf.split("\n")[1].trim().endsWith(" Topic 9") && buf.split("\n")[2].trim().endsWith(" Topic 9"));
	}

	@Test
	public void S_wBad1_Test_18() {
		System.out.println("Test 18");
		c3.send("W ");
		sleep(1);
		String buf = c3.getBuffer();
		System.out.println("C3 sendet W_");
		System.out.println("IS: " + buf + " SHOULD: E");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buf.startsWith("E "));
	}

	@Test
	public void T_tBad1_Test_19() {
		System.out.println("Test 19");
		c3.send("T ");
		sleep(1);
		String buf = c3.getBuffer();
		System.out.println("C3 sendet T_");
		System.out.println("IS: " + buf + " SHOULD: 0 / E");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buf.startsWith("E ") || buf.trim().equals("0"));
	}

	@Test
	public void U_tBad2_Test_20() {
		System.out.println("Test 20");
		c3.send("T WUP");
		sleep(1);
		String buf = c3.getBuffer();
		System.out.println("C3 sendet T WUP");
		System.out.println("IS: " + buf + " SHOULD: 0 / E");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buf.startsWith("E ") || buf.trim().equals("0"));
	}

	@Test
	public void V_Unkown_Test_21() {
		System.out.println("Test 21");
		c3.send("Bla");
		sleep(1);
		String buf = c3.getBuffer();
		System.out.println("C3 sendet Bla");
		System.out.println("IS: " + buf + " SHOULD: E");
		System.out.println("--------------------------------------------------------------------------------------");

		assertTrue(buf.startsWith("E "));
	}

	@Test
	public void W_SpammyDis_Test_22() {
		System.out.println("Test 22");
		for (int i = 0; i < 100; i++) {
			c1.sendB("P\n1\n2\n0 Topic 0\nText 0");
			c2.sendB("P\n1\n3\n1 Topic 1\nText 1\nText 1,2");
			c3.send("L");
			c3.sendB("P\n1\n4\n2 Topic 2\nText 2\nText 2,2\nText 2,3");
			c3.sendB("P\n1\n2\n1 Topic 8\nText 8");
			c1.send("W 10");
			c1.sendB("P\n2\n2\n3 Topic 3\nText 3\n2\n4 Topic 4\nText 4");
			c3.send("T Topic 8");
			c2.sendB("P\n3\n2\n5 Topic 5\nText 5\n2\n6 Topic 6\nText 6\n2\n7 Topic 7\nText 7");
			c3.sendB("P\n2\n2\n8 Topic 8\nText 8\n3\n9 Topic 9\nText 9\nText 9,2");
		}
		sleep(5);
		c3.send("X");
		sleep(1);
		System.out.println("C3 sendet X");
		try {
			int tmp = c3.cSocket.getInputStream().read();
			System.out.println("IS: " + tmp + " SHOULD: -1");
			assertTrue(tmp == -1);
		} catch (IOException e) {
			System.out.println("Exeption occured");
			fail("IOException during KeepAlive");
		}
	}

	private static void sleep(int t) {
		try {
			Thread.sleep(t * 1000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
}